<?php
/*--------------------------------------------------------------------------------------------------
    ThemeDeleterService.php 2020-01-29
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2020 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\Components\Theme\Services;

use Exception;
use FileNotFoundException;
use Gambio\StyleEdit\Adapters\Interfaces\CacheCleanerInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\BasicThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Exceptions\UndeletableThemeException;
use Gambio\StyleEdit\Core\Components\Theme\Repositories\StyleEditThemeRepository;
use Gambio\StyleEdit\Core\Components\Theme\Repositories\ThemeConfigurationRepository;
use Gambio\StyleEdit\Core\TranslatedException;

/**
 * Class ThemeDeleterService
 * @package Gambio\StyleEdit\Core\Components\Theme\Services
 */
class ThemeDeleterService
{
    /**
     * @var CacheCleanerInterface
     */
    private $cacheCleaner;
    /**
     * @var ThemeConfigurationRepository
     */
    private $configurationRepository;
    /**
     * @var StyleEditThemeRepository
     */
    private $repository;
    
    
    /**
     * ThemeDeleterService constructor.
     *
     * @param ThemeConfigurationRepository $configurationRepository
     * @param StyleEditThemeRepository     $repository
     * @param CacheCleanerInterface        $cacheCleaner
     */
    public function __construct(
        ThemeConfigurationRepository $configurationRepository,
        StyleEditThemeRepository $repository,
        CacheCleanerInterface $cacheCleaner
    ) {
        $this->configurationRepository = $configurationRepository;
        $this->repository              = $repository;
        $this->cacheCleaner            = $cacheCleaner;
    }
    
    
    /**
     * @param BasicThemeInterface $theme
     *
     * @throws FileNotFoundException
     * @throws TranslatedException
     * @throws UndeletableThemeException
     */
    public function deleteTheme(BasicThemeInterface $theme): void
    {
        $theme = $this->configurationRepository->getById($theme->id());
        if (!$theme->isPreview()) {
            if ($theme->isActive()) {
                throw new UndeletableThemeException('Is not possible to delete the active theme!');
            }
        
            if (count($theme->children()) > 0) {
                throw new UndeletableThemeException('Theme has children!');
            }
        
            if (!$theme->isRemovable()) {
                throw new UndeletableThemeException('The theme cannot be deleted!');
            }
            $this->deletePreviewOf($theme);
        }
    
        $this->repository->delete($theme);
        $this->cacheCleaner->clearShopCache();
    }
    
    
    /**
     * @param BasicThemeInterface $themeId
     */
    protected function deletePreviewOf(BasicThemeInterface $themeId): void
    {
        try {
            $theme = $this->configurationRepository->getById($themeId->id() . '_preview');
            if ($theme) {
                $this->repository->delete($theme);
            }
        } catch (Exception $e) {
            //ignore exception
        }
    }
    
    
}