<?php
/*--------------------------------------------------------------------------------------------------
    ProductListWidgetOutputCommand.php 2019-12-02
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2019 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

/**
 * Class ProductListWidgetOutputCommand
 */
class ProductListWidgetOutputCommand
{
    /**
     * @var ProductReadService
     */
    protected $productReadService;

    /**
     * @var VPEReadService
     */
    protected $vpeReadService;

    /**
     * @var ProductsShippingStatusSource
     */
    protected $productsShippingStatusSource;

    /**
     * @var xtcPrice_ORIGIN|mixed
     */
    protected $xtcPrice;
    /**
     * @var FeaturedProductReadService
     */
    protected $featuredProductReadService;

    /**
     * @var array
     */
    protected $products;
    /**
     * @var string
     */
    protected $elementId;
    /**
     * @var string
     */
    protected $presentation;
    /**
     * @var LanguageCode
     */
    protected $languageCode;
    /**
     * @var int
     */
    protected $maxProducts;

    /**
     * @var string
     */
    protected $elementClassName;
    protected $random;
    protected $listType;
    protected $customerGroupId;
    protected $isFsk18Purchasable;
    protected $categoryId;
    /**
     * @var IdType
     */
    protected $languageId;

    const SLIDES_PER_PREVIEW = [
        1 => 12,
        2 => 6,
        3 => 4,
        4 => 3,
        6 => 2,
        12 => 1
    ];


    /**
     * Create ProductListWidgetOutputCommand instance
     *
     * @param int $categoryId|null
     * @param string $listType
     * @param bool $random
     * @param int $maxProducts
     * @param string $elementId
     * @param string $elementClassName
     * @param array $products
     * @param LanguageCode $languageCode
     * @param IdType $languageId
     * @param string $presentation|null
     * @param ProductReadService|null $productReadService Product read service
     * @param VPEReadService|null $vpeReadService VPE read service
     * @param ProductsShippingStatusSource|null $productsShippingStatusSource Product shipping status source
     * @param xtcPrice_ORIGIN|null $xtcPrice Pricing format class
     * @param FeaturedProductReaderInterface $featuredProductReadService
     */
    public function __construct(
        ?int $categoryId,
        string $listType,
        bool $random,
        int $maxProducts,
        array $products,
        string $elementId,
        string $elementClassName = '',
        IdType $languageId,
        LanguageCode $languageCode,
        ?string $presentation = 'slider',
        ProductReadService $productReadService = null,
        VPEReadService $vpeReadService = null,
        ProductsShippingStatusSource $productsShippingStatusSource = null,
        xtcPrice_ORIGIN $xtcPrice = null,
        FeaturedProductReaderInterface $featuredProductReadService = null
    )
    {
        $this->customerGroupId = new IntType($_SESSION['customers_status']['customers_status_id'] ?? 0);
        $this->isFsk18Purchasable = new BoolType($_SESSION['customers_status']['customers_fsk18_display'] === '1');

        $this->categoryId = $categoryId;
        $this->listType = $listType;
        $this->random = $random;
        $this->maxProducts = $maxProducts;
        $this->products = $products;
        $this->elementId = $elementId;
        $this->elementClassName = $elementClassName;
        $this->languageCode = $languageCode;
        $this->languageId = $languageId;
        $this->presentation = $presentation ?? 'slider';
        $this->productReadService = $productReadService ?? StaticGXCoreLoader::getService('ProductRead');
        $this->vpeReadService = $vpeReadService ?? StaticGXCoreLoader::getService('VPERead');
        $this->featuredProductReadService = $featuredProductReadService ??
            StaticGXCoreLoader::getService('FeaturedProductRead');
        $this->productsShippingStatusSource = $productsShippingStatusSource ??
            MainFactory::create(ProductsShippingStatusSource::class);

        $this->xtcPrice = $xtcPrice ?? MainFactory::create(xtcPrice_ORIGIN::class,
                $_SESSION['currency'],
                $_SESSION['customers_status']['customers_status_id']);
    }


    /**
     * Get the HTML output
     *
     * @return string HTML fragment
     * @throws EntityNotFoundException
     */
    public function execute()
    {
        $template = ($this->presentation === 'list') ? "product_listing.html" : "product_listing_swiper.html";

        $products = $this->productsByListType($this->listType);

        $productData = [];

        foreach ($products as $product) {
            if ($product instanceof ProductListItem) {
                $productData[] = $this->dataFromProductListItem($product);
                continue;
            }

            if ($product instanceof StoredProduct) {
                $productData[] = $this->dataFromStoredProduct($product);
                continue;
            }

            if ($product instanceof FeaturedProduct) {
                $productData[] = $this->dataFromFeaturedProduct($product, $this->languageCode);
            }
        }
        if (count($productData)) {
            return ProductListContentViewGenerator::generate(
                $productData,
                $this->elementId,
                $template,
                $this->elementClassName,
                $this->getSwiperBreakpoints()
            );
        } else {
            return '';
        }
    }


    /**
     * Return an array of product IDs based on the selected list type and provided language
     *
     * @param string $listType List type
     *
     * @return array
     */
    protected function productsByListType(string $listType): array
    {

        $featuredProductSettings = MainFactory::create(FeaturedProductSettings::class,
            new IntType($this->maxProducts),
            new BoolType($this->random),
            $this->customerGroupId,
            $this->isFsk18Purchasable,
            $this->languageCode);

        $products = [];

        if ($listType === 'own-list') {
            return $this->getOwnListProducts();
        }

        switch ($listType) {
            case 'category':
                $categoryId = $this->categoryId;
                if ($categoryId) {
                    try {
                        $products = $this->featuredProductReadService->getProductsByCategoryId($featuredProductSettings,
                            new IntType($categoryId))
                            ->getArray();
                    } catch (Exception $e) {
                        //hide exceptions in order to keep stability
                    }
                }
                break;
            case 'deals':
                $products = $this->featuredProductReadService
                    ->getOfferedProducts($featuredProductSettings)
                    ->getArray();
                break;
            case 'recommended':
                $products = $this->featuredProductReadService
                    ->getTopProducts($featuredProductSettings)
                    ->getArray();
                break;
            case 'coming-soon':
                $products = $this->featuredProductReadService
                    ->getUpcomingProducts($featuredProductSettings)
                    ->getArray();
                break;
            case 'new':
                $products = $this->featuredProductReadService
                    ->getNewProducts($featuredProductSettings)
                    ->getArray();
                break;
        }

        return $products;
    }


    /**
     * Create data array from ProductListItem object
     *
     * @param ProductListItem $product Product
     *
     * @return array
     * @throws EntityNotFoundException
     */
    protected function dataFromProductListItem(ProductListItem $product): array
    {

        $data = [
            'showManufacturerImages' => 'true',
            'showProductRibbons' => 'true',
            'PRODUCTS_SHIPPING_NAME' => $this->productsShippingStatusSource->get_shipping_status_name($product->getShippingTimeId(),
                $this->languageId->asInt()),
            'PRODUCTS_SHORT_DESCRIPTION' => $product->getProductObject()->getShortDescription($this->languageCode),
            'PRODUCTS_META_DESCRIPTION' => $product->getName(),
            'PRODUCTS_NAME' => $product->getName(),
            'PRODUCTS_IMAGE' => $this->getImageCorrectPath($product->getImage()),
            'PRODUCTS_IMAGE_ALT' => $product->getImageAltText(),
            'PRODUCTS_ID' => $product->getProductId(),
            'PRODUCTS_VPE' => '',
            'PRODUCTS_LINK' => xtc_href_link(FILENAME_PRODUCT_INFO,
                xtc_product_link($product->getProductId(),
                    $product->getName())),
            'PRODUCTS_PRICE' => $this->getProductPrice($product)
        ];

        if ($product->isVpeActive()) {
            $data['PRODUCTS_VPE'] = $this->getProductsVpeValue($product->getProductId());
        }

        return $data;
    }


    /**
     * Create data array from StoredProduct object
     *
     * @param StoredProduct $product Product
     *
     * @return array
     * @throws EntityNotFoundException
     */
    protected function dataFromStoredProduct(StoredProduct $product): array
    {
        $data = [
            'showManufacturerImages' => 'true',
            'showProductRibbons' => 'true',
            'PRODUCTS_SHIPPING_NAME' => $this->productsShippingStatusSource->get_shipping_status_name($product->getShippingTimeId(),
                $this->languageId->asInt()),
            'PRODUCTS_SHORT_DESCRIPTION' => $product->getShortDescription($this->languageCode),
            'PRODUCTS_META_DESCRIPTION' => $product->getName($this->languageCode),
            'PRODUCTS_NAME' => $product->getName($this->languageCode),
            'PRODUCTS_IMAGE' => $this->getImageCorrectPath($product->getPrimaryImage()->getFilename()),
            'PRODUCTS_IMAGE_ALT' => $product->getPrimaryImage()->getAltText($this->languageCode),
            'PRODUCTS_ID' => $product->getProductId(),
            'PRODUCTS_VPE' => '',
            'PRODUCTS_LINK' => xtc_href_link(FILENAME_PRODUCT_INFO,
                xtc_product_link($product->getProductId(),
                    $product->getName($this->languageCode))),
            'PRODUCTS_PRICE' => $this->getProductPrice($product)
        ];

        if ($product->isVpeActive()) {
    
            $data['PRODUCTS_VPE'] = $this->getProductsVpeValue($product->getProductId());
        }

        return $data;
    }
    
    
    /**
     * @param int $productsId
     *
     * @deprecated With variant products this method will be unnecessary und the 2 product widgets need a refactoring
     * @return string
     */
    public function getProductsVpeValue(int $productsId): string
    {
        $productObj  = MainFactory::create('product', $productsId);
        $productData = $productObj->buildDataArray($productObj->data);
    
        return $productData['PRODUCTS_VPE'] ?? '';
    }
    
    /**
     * Create data array from FeaturedProduct object
     *
     * @param FeaturedProduct $product Product
     * @param LanguageCode $languageCode Language
     *
     * @return array
     * @throws EntityNotFoundException
     */
    protected function dataFromFeaturedProduct(FeaturedProduct $product, LanguageCode $languageCode): array
    {

        $data = [
            'showManufacturerImages' => 'true',
            'showProductRibbons' => 'true',
            'PRODUCTS_SHIPPING_NAME' => $product->getShippingStatusName(),
            'PRODUCTS_SHORT_DESCRIPTION' => $product->getShortDescription(),
            'PRODUCTS_META_DESCRIPTION' => $product->getMetaDescription(),
            'PRODUCTS_NAME' => $product->getName(),
            'PRODUCTS_IMAGE' => $this->getImageCorrectPath($product->getImage()),
            'PRODUCTS_IMAGE_ALT' => $product->getImageAltText(),
            'PRODUCTS_ID' => $product->getProductsId(),
            'PRODUCTS_VPE' => $product->getVpeID(),
            'PRODUCTS_LINK' => xtc_href_link(FILENAME_PRODUCT_INFO,
                xtc_product_link($product->getProductsId(),
                    $product->getName())),
            'PRODUCTS_PRICE' => $this->getProductPrice($product)
        ];

        if ($product->getVpeID() > 0) {
            $data['PRODUCTS_VPE'] = $this->getProductsVpeValue($product->getProductsId());
        }

        return $data;
    }


    /**
     * @return array
     */
    protected function getOwnListProducts(): array
    {
        $productIds = $this->products;

        if ($this->random === true) {
            shuffle($productIds);
        }

        if (count($productIds) > $this->maxProducts) {
            $productIds = array_slice($productIds, 0, $this->maxProducts);
        }

        $products = [];

        foreach ($productIds as $id) {
            try {
                $products[] = $this->productReadService->getProductById(MainFactory::create(IdType::class, $id));
            } catch (Exception $e) {
                //hide error messages in order to keep stability
            }
        }

        return $products;
    }


    /**
     * @param $image
     *
     * @return string
     */
    protected function getImageCorrectPath($image): string
    {
        return (ENABLE_SSL ? HTTPS_SERVER : HTTP_SERVER) . DIR_WS_CATALOG
            . "/images/product_images/thumbnail_images/{$image}";
    }

    /**
     * @return array
     */
    protected function getSwiperBreakpoints(): array
    {
        $colXs = $GLOBALS['coo_template_control']->findSettingValueByName('gx-product-listing-col-xs');
        $colSm = $GLOBALS['coo_template_control']->findSettingValueByName('gx-product-listing-col-sm');
        $colMd = $GLOBALS['coo_template_control']->findSettingValueByName('gx-product-listing-col-md');
        $colLg = $GLOBALS['coo_template_control']->findSettingValueByName('gx-product-listing-col-lg');

        return [
            [
                'breakpoint' => 40,
                'usePreviewBullets' => true,
                'slidesPerView' => self::SLIDES_PER_PREVIEW[$colXs] ?? 2,
                'centeredSlides' => true
            ],
            [
                'breakpoint' => 60,
                'usePreviewBullets' => true,
                'slidesPerView' => self::SLIDES_PER_PREVIEW[$colSm] ?? 3,
            ],
            [
                'breakpoint' => 80,
                'usePreviewBullets' => true,
                'slidesPerView' => self::SLIDES_PER_PREVIEW[$colMd] ?? 2,
            ],
            [
                'breakpoint'        => 100,
                'usePreviewBullets' => true,
                'slidesPerView'     => self::SLIDES_PER_PREVIEW[$colLg] ?? 5, // default slides per view
            ]
        ];
    }

    /**
     * @param $product
     * @return string
     */
    protected function getProductPrice($product): string
    {
        $productId = method_exists(get_class($product), 'getProductId') ?
            $product->getProductId() :
            $product->getProductsId();

        $propertiesControl = MainFactory::create_object('PropertiesControl');
        $productCombination = $propertiesControl->get_cheapest_combi($productId, $this->languageCode);

        $price = $this->xtcPrice->xtcGetPrice(
            $productId,
            true,
            $product->getQuantity(),
            $product->getTaxClassId(),
            $product->getPrice(),
            1,
            0,
            true,
            true,
            $productCombination['products_properties_combis_id'],
            true
        );

        return $price['formated'] ?? 0;
    }

}
